/*---------------------------------------------------------------------------\
| Project:          Demo LV2 PIC18F458 CAN node firmware
|
| File:             main.c
|
| Description:      Mainline routines.
|
| Global Routines:  Main()
|
| Local Routines:   None
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 06/01/2002 | Stolen from ADG code.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

#include <piccore.h>   // Includes for the PicCore framework

#if CORE_VERSION != 5
#error HEY! The core version has changed! Make SURE your interfaces match!
#endif

#include "demo.h"   // Includes for the demo firmware program



//------------------------------------------------------------------------
//  Declarations of local routines
//------------------------------------------------------------------------

void Initialize (void);


//------------------------------------------------------------------------
//  Global Variables
//------------------------------------------------------------------------

 bit    timer2Tick;



/*---------------------------------------------------------------------------\
| Function:     main
|
| Purpose:      Demo the LV2 Core Firmware
|
| Discussion:   Sends out CAN messages at some frequency, and regurgitates
|               any received messages.
|
| Expects:      -
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 08/10/2002 | Modified to core version 4. How cool is that?
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void main (void)
{
    static CanMessage_t tempMsg;
    static CanMessage_t tempMsg2;

    // Initialize everything and its brother

    Initialize();

    // TEST TEST TEST

    tempMsg.header.all = (0x0200 << 5);
    tempMsg.header.bits.length = 1;
    tempMsg.data[0] = 0x00;

    SendCanMessage(&tempMsg);


    // Loop for ever...and......ev....er....and.......ev............er.

    while (TRUE)
    {
        if (timer2Tick)
        {
            ++tempMsg.data[0];
            SendCanMessage(&tempMsg);
            timer2Tick = 0;
        }

        if (canMessageAvailable)
        {
            GetCanMessage(&tempMsg2);
            SendCanMessage(&tempMsg2);
        }
    }

}




/*---------------------------------------------------------------------------\
| Function:     Initialize
|
| Purpose:      Initialize everything. Really.
|
| Discussion:   Did I mention initialization?
|
| Expects:      To be called after a RESET - eg, registers reset to known
|               values and and interrupts off.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void Initialize (void)
{

    // Setting up the CAN bus peripheral: We're passing constants
    // to the initialization routine so that we can just set the
    // initialize initialization :) here.

    const CanHeader_t canMaskB0 = (0x07FF << 5);
    const CanHeader_t canMaskB1 = (0x07FF << 5);
    const CanHeader_t CanFilterB0F0 = (0x07FC <<5);
    const CanHeader_t CanFilterB0F1 = (0x0201 << 5);
    const CanHeader_t CanFilterB1F2 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F3 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F4 = (0x0000 << 5);
    const CanHeader_t CanFilterB1F5 = (0x0000 << 5);

    // Get the outputs stabilized FIRST.

    InitializePorts (  0b00000000,           // port_a_latch
                       0b00000000,           // port_a_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RA0/AN0/CVref     = out =
                      // ||||||+------------ RA1/AN1           = out =
                      // |||||+------------- RA2/AN2/Vref-     = out =
                      // ||||+-------------- RA3/AN3/Vref+     = in  =
                      // |||+--------------- RA4/T0CKI         = out =
                      // ||+---------------- RA5/AN4/SS*/LVDIN = out =
                      // |+----------------- RA6/OSC2/CLKOUT   = out =
                      // +------------------ <unimplemented>
                      // WARNING: DON'T FORGET TO INTIALIZE THE ADC CORRECTLY
                      // TO ENABLE PORT A TO BE OUTPUTS.

                       0b00000000,           // port_b_latch
                       0b00001000,           // port_b_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RB0/INT0          = out =
                      // ||||||+------------ RB1/INT1          = out =
                      // |||||+------------- RB2/CANTX         = out = CANTX
                      // ||||+-------------- RB3/CANRX         = in  = CANRX
                      // |||+--------------- RB4               = out =
                      // ||+---------------- RB5/PGM           = out =
                      // |+----------------- RB6/PGC           = out = ICD2 PGC
                      // +------------------ RB7/PGD           = out = ICD2 PGD

                       0b00000000, // port_c_latch
                       0b00000000, // port_c_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RC0/T1OSO/T1CKI   = out =
                      // ||||||+------------ RC1/T1OSI         = out =
                      // |||||+------------- RC2/CCP1X         = out =
                      // ||||+-------------- RC3/SCK/SCL       = in  =
                      // |||+--------------- RC4/SDI/SDA       = out =
                      // ||+---------------- RC5/SDO           = out =
                      // |+----------------- RC6/TX/CK         = out =
                      // +------------------ RC7/RX/DT         = out =

                       0b00000000, // port_d_latch
                       0b00000000, // port_d_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RD0/PSP0/C1IN+    = out =
                      // ||||||+------------ RD1/PSP1/C1IN-    = out =
                      // |||||+------------- RD2/PSP2/C2IN+    = out =
                      // ||||+-------------- RD3/PSP3/C2IN-    = in  =
                      // |||+--------------- RD4/PSP4/ECCP1/P1A= out =
                      // ||+---------------- RD5/PSP5/P1B      = out =
                      // |+----------------- RD6/PSP6/P1C      = out =
                      // +------------------ RD7/PSP7/P1D      = out =

                       0b00000000, // port_e_latch
                       0b00000000, // port_e_direction (1 = in, 0 = out)
                      // ||||||||
                      // |||||||+----------- RE0/RD*/AN5       = out =
                      // ||||||+------------ RE1/WR*/AN6/C1OUT = out =
                      // |||||+------------- RE2/CS*/AN7/C2OUT = out =
                      // ||||+-------------- <unimplimented>
                      // |||+--------------- <unimplimented>
                      // ||+---------------- <unimplimented>
                      // |+----------------- <unimplimented>
                      // +------------------ <unimplimented>

                       ( PORTS_PORTB_PULLUPS_OFF
                         | PORTS_INT0_RISING_EDGE
                         | PORTS_INT1_RISING_EDGE
                         | PORTS_INT0_INTS_DISABLED
                         | PORTS_INT1_INTS_DISABLED ) );


    // Initialize the ADC peripheral. Note that it directly affects PORT A
    // and PORT E pins due to the ADC taking over the TRIS register.

    initialize_adc ( ADC_DISABLED
                     | ADC_PORT_OFF
                     | ADC_INTS_DISABLED );

    // Turn on serial transmitter and receiver. Turn on receiver interrupts
    // but leave the transmitter interrupts off (we'll turn them on when
    // there's actual data to send over the port). Make both interrupts
    // low priority, and se the bit rate to 9600bps.

    InitializeSerial (  SERIAL_TRANSMIT_ENABLE
                        | SERIAL_TRANSMIT_DISABLE_INTS
                        | SERIAL_RECEIVE_ENABLE
                        | SERIAL_RECEIVE_DISABLE_INTS
                        | SERIAL_BITRATE_9600 );

    // Turn on timer2, HP interrupts @ 1KHz

    initialize_timer2 (  TIMER2_DISABLE_INTS
                        | TIMER2_1KHZ );

    // Turn on CAN: See const variable declarations above
    // for the values. Note that as soon as we initialize the
    // the bus, we can start to receive messages, so be sure
    // that interrupts are enabled VERY SOON after this call
    // (No more than about a  message length, which at top
    // speed is about 1/22KHz = 45us which at 20MHz is only
    // 250 instructions.

    InitializeCan ( CAN_BITRATE_1MBPS,
                    canMaskB0,          // Mask RXB0
                    canMaskB1,          // Mask RXB1
                    CanFilterB0F0,      // Filter RXB0 F0
                    CanFilterB0F1,      // Filter RXB0 F1
                    CanFilterB1F2,      // Filter RXB1 F2
                    CanFilterB1F3,      // Filter RXB1 F3
                    CanFilterB1F4,      // Filter RXB1 F4
                    CanFilterB1F5);     // Filter RXB1 F5


    // ---------------------------------------------------------------------
    // PICCORE: ALL INITIALIZATION THAT MUST BE DONE PRIOR TO INTERRUPTS
    // BEING ENABLED MUST BE DONE PRIOR TO "InitializeInterrupts()".

    InitializeInterrupts();

    // PICCORE: INTERRUPTS ARE INITIALIZED; FEEL FREE TO INITIALIZE THINGS
    // WHICH REQUIRE INTERRUPTS TO BE RUNNING AFTER THIS POINT.
    // ---------------------------------------------------------------------
}



/*---------------------------------------------------------------------------\|
| Descr.:   Application specific code which must be executed on an interrupt.
|           Essentially, these are "hooks" into the core's interrupt service
|           routines.
|
|           YOU SHOULD MODIFIY THIS FILE FOR YOUR APPLICATION, ALONG WITH
|           THE CONFIG.H FILE. FEEL FREE TO EVEN CUT AND COPY THESE FUNCTIONS
|           TO OTHER FILES.
|
|           HOWEVER, DO *NOT* CHANGE THE #IF, OR THE NAME AND TYPE OF THE
|           HANDLER FUNCTIONS.
\---------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------\
| Function:     timer2_application_handler
|
| Purpose:      Applicationh specific handler for Timer2 interrupt
|
| Discussion:   APPLICATION: INSERT DESCRIPTION HERE
|
|
| Expects:      -
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 08/10/2002 | Created.
\---------------------------------------------------------------------------*/

#if INTS_TIMER2_APPLICATION_HOOK == INTS_HOOK_ENABLED

void timer2_application_handler (void)
{

    timer2Tick = 1;

}

#endif


/*---------------------------------------------------------------------------\
| Function:     int0_application_handler
|
| Purpose:      Applicationh specific handler for INT0 (PORT B, pin RB0)
|               interrupt.
|
| Discussion:   APPLICATION: INSERT DESCRIPTION HERE
|
|
| Expects:      -
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 08/10/2002 | Created.
\---------------------------------------------------------------------------*/

#if INTS_INT0_APPLICATION_HOOK == INTS_HOOK_ENABLED

void int0_application_handler (void)
{

    asm("NOP");

}

#endif



/*---------------------------------------------------------------------------\
| Function:     adc_application_handler
|
| Purpose:      Applicationh specific handler for the PIC's 10bit ADC
|               interrupt-on-conversion-done.
|
| Discussion:   APPLICATION: INSERT DESCRIPTION HERE
|
|
| Expects:      -
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 08/10/2002 | Created.
\---------------------------------------------------------------------------*/

#if INTS_ADC_APPLICATION_HOOK == INTS_HOOK_ENABLED

void adc_application_handler (void)
{

    asm("NOP");

}

#endif


