/*---------------------------------------------------------------------------\
| 18F458 DEMO SOFTWARE
|
| File:                 timer2.c          All Timer2 Code
|
| Global Routines:      initialize_timer2()
|                       timer2_isr()
|
| Local Routines:       None
|
| Discussion:           Sure.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created. Again.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes and local #defines
//------------------------------------------------------------------------

#include <piccore.h>   // Includes for the project (includes other includes!)


//------------------------------------------------------------------------
//  Global Variables
//------------------------------------------------------------------------

// None

//------------------------------------------------------------------------
//  Module-wide variables
//------------------------------------------------------------------------

// None


/*---------------------------------------------------------------------------\
| Function:     initialize_timer2
|
| Purpose:      Guess.
|
| Discussion:   Huh?
|
| Expects:      All interupts to be off (after all, we were just reset).
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void initialize_timer2 (enum TIMER2_INIT_MASK timer2_options)
{

    // On reset, Timer2 is set to the following RESET values:
    //
    // T2CON = 0b00100101
    //           ||||||||
    //           |||||||+----------- T2CKPS0 = 00 = prescaler 1:1
    //           ||||||+------------ T2CKPS1
    //           |||||+------------- TMR2ON  = 0 = Timer2 off
    //           ||||+-------------- TOUTPS0 = 0000 = postscaler 1:1
    //           |||+--------------- TOUTPS1
    //           ||+---------------- TOUTPS2
    //           |+----------------- TOUTPS3
    //           +------------------ <Unimplemented on PIC>
    //
    //  PR2 = 0xFF

    switch (timer2_options & 0b00000111)
    {

        case TIMER2_OFF:            // Case 0 = Off!
            TMR2ON = 0;             // Turn off the whole timer module
            break;

        case TIMER2_250HZ:          // Case 1 = 250Hz
            PR2 = 249;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101110;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_500HZ:          // Case 2 = 500Hz
            PR2 = 249;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b01010101;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_1KHZ:           // Case 3 = 1KHz
            PR2 = 249;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101101;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_2KHZ:           // Case 4 = 2KHz
            PR2 = 124;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101101;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_2_5KHZ:         // Case 5 = 2.5KHz
            PR2 = 24;               // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101110;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_5KHZ:           // Case 6 = 5KHz
            PR2 = 249;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00001101;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_10KHZ:          // Case 7 = 10KHz
            PR2 = 99;               // ASSUMES FOSC = 20MHZ
            T2CON |= 0b01001100;    // ASSUMES FOSC = 20MHZ
            break;

        default:                    // DEFAULT: 2.5KHz
            PR2 = 24;               // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101110;    // ASSUMES FOSC = 20MHZ
            ThrowWarning(WARN_TIMER2_RATE);
    }

    // Choose the interrupt priority based on the config.h settings

    #if INTS_TIMER2_INT_PRIORITY == INTS_HIGH_PRIORITY_INTERRUPT
    Timer2PrioritySetting = HIGH_PRIORITY_INT;
    #else
    Timer2PrioritySetting = LOW_PRIORITY_INT;
    #endif

    // Enable the interrupts if choosen

    if(timer2_options & TIMER2_ENABLE_INTS)
        enable_timer2_ints();
    else
        disable_timer2_ints();
}



/*---------------------------------------------------------------------------\
| Function:     SetTimer2BaseRate
|
| Purpose:      Set the Timer2 period.
|
| Discussion:   Huh?
|
| Expects:      All interupts to be off (after all, we were just reset).
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void set_timer2_base_rate (enum TIMER2_INIT_MASK timer2_options)
{

    switch (timer2_options & 0b00000111)
    {

        case TIMER2_OFF:            // Case 0 = Off!
            TMR2ON = 0;             // Turn off the whole timer module
            break;

        case TIMER2_250HZ:          // Case 1 = 250Hz
            PR2 = 249;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101110;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_500HZ:          // Case 2 = 500Hz
            PR2 = 249;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b01010101;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_1KHZ:           // Case 3 = 1KHz
            PR2 = 249;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101101;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_2KHZ:           // Case 4 = 2KHz
            PR2 = 124;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101101;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_2_5KHZ:         // Case 5 = 2.5KHz
            PR2 = 24;               // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101110;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_5KHZ:           // Case 6 = 5KHz
            PR2 = 249;              // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00001101;    // ASSUMES FOSC = 20MHZ
            break;

        case TIMER2_10KHZ:          // Case 7 = 10KHz
            PR2 = 99;               // ASSUMES FOSC = 20MHZ
            T2CON |= 0b01001100;    // ASSUMES FOSC = 20MHZ
            break;

        default:                    // DEFAULT: 2.5KHz
            PR2 = 24;               // ASSUMES FOSC = 20MHZ
            T2CON |= 0b00101110;    // ASSUMES FOSC = 20MHZ
            ThrowWarning(WARN_TIMER2_RATE);
    }
}

/*---------------------------------------------------------------------------\
| Function:     timer2_isr
|
| Purpose:      Handle the Timer 2 interrupt.
|
| Discussion:   Clears the Timer2 interrupt and if an interrupt handler exists
|               (the function pointer is non-null) then it calls the handler.
|               Note that if the Timer 2 interrupts are turned off, this
|               function goes away (and thus never has the opportunity to
|               vector off to la-la land.
|
| Expects:      -
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 08/10/2002 | Added application specific interrup handler via function ptr.
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

#if INTS_TIMER2_ISR_USED == INTS_CORE_ISR_ENABLED

void timer2_isr (void)
{
    clear_timer2_interrupt();

    // If an application specific interrupt handler is defined,
    // then call it. See init_int.h.

    #if INTS_TIMER2_APPLICATION_HOOK == INTS_HOOK_ENABLED
    timer2_application_handler();
    #endif
}

#endif

